@@ -28,8 +28,9 @@
 // END OF LICENSE HEADER
 
 // Extract youtube fmt parameter
-LinternaMagica.prototype.extract_youtube_fmt_parameter = function(data)
+LinternaMagica.prototype.extract_youtube_fmt_parameter = function()
 {
+    var data = this.script_data;
     var fmt_re = new RegExp (
 	"(\\\"|\\\'|\\\&)fmt_list"+
 	    "(\\\"|\\\')*(\\\=|\\\:|,)\\\s*(\\\"|\\\')*"+
@@ -155,7 +156,7 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
 		break;
 	    case '13':
 	    case '17':
-		nore_info += "MPEG-4 Visual, AAC";
+		more_info += "MPEG-4 Visual, AAC";
 		break;
 	    default:
 		more_info += " " + this._("Unkown encoding");
@@ -195,7 +196,7 @@ LinternaMagica.prototype.detect_youtube_flash_upgrade = function(object_data)
 
     // Fancy flash upgrade message. The element with id flash-upgrade
     // might be obsolete now.
-    var watch_player = document.getElementById("watch-player");
+    var watch_player = document.getElementById("watch7-player");
     var fancy_alert = null;
 
     if(watch_player && watch_player.hasAttribute("class") &&
@@ -211,12 +212,12 @@ LinternaMagica.prototype.detect_youtube_flash_upgrade = function(object_data)
     }
     
 
-    // With default timeout 500mS this will be 6 sec. Stop checking and insert.    
+    // With default timeout 2000mS this will be 10 sec. Stop checking and insert.    
     // Might be flashblock
-    if (document.getElementById("flash-upgrade") ||
+    if (document.getElementById("movie_player") ||
 	document.getElementById("movie_player-html5") ||
 	fancy_alert ||
-	this.youtube_flash_upgrade_counter >= 12 )
+	this.youtube_flash_upgrade_counter >= 5 )
     {
 	clearInterval(this.youtube_flash_upgrade_timeout);
 
@@ -228,13 +229,14 @@ LinternaMagica.prototype.detect_youtube_flash_upgrade = function(object_data)
 	this.log("LinternaMagica.detect_youtube_flash_upgrade:\n"+
 		 "Creating video object.",2);
 
-	this.create_video_object(object_data);
+	setTimeout(this.create_video_object(object_data), 1000);
     }
 }
 
 // Extract links data for youtube from fmt_url_map
-LinternaMagica.prototype.extract_youtube_fmt_url_map = function(data)
+LinternaMagica.prototype.extract_youtube_fmt_url_map = function()
 {
+    var data = this.script_data;
     var fmt_re = new RegExp (
 	"(\\\"|\\\'|\\\&|\\\&amp;)url_encoded_fmt_stream_map"+
 	    "(\\\"|\\\')*(\\\=|\\\:|,)\\\s*(\\\"|\\\')*"+
@@ -248,6 +250,9 @@ LinternaMagica.prototype.extract_youtube_fmt_url_map = function(data)
 	this.log("LinternaMagica.extract_youtube_fmt_url_map:\n"+
 		 "Extracted fmt_url_map.",1);
 
+	this.log("LinternaMagica.extract_youtube_fmt_url_map:\n"+
+		 "RAW map variable:"+fmt,5);
+
 	// Hash with keys fmt_ids and values video URLs
 	var map = new Object();
 
@@ -262,14 +267,16 @@ LinternaMagica.prototype.extract_youtube_fmt_url_map = function(data)
 	    // (itag=fmt_id)*url=URL&type=video/...&(itag=fmt_id)*
 	    var link = fmt[url].match(/url=([^&]+)/);
 	    var fmt_id = fmt[url].match(/itag=([0-9]+)/);
-
+	    var sig = fmt[url].replace(/\\u0026/g, '&').match(/sig=[^&]+/);
+		sig = sig[sig.length-1].replace(/sig/,'signature');
+	    
 	    if (fmt_id && link)
 	    {
 		links++;
 		link = unescape(link[link.length-1]);
 		link = link.split(/\\u0026/)[0];
 
-		map[fmt_id[fmt_id.length-1]] =  link;
+		map[fmt_id[fmt_id.length-1]] =  link+"&"+sig;
 	    }
 	}
 
@@ -327,6 +334,14 @@ LinternaMagica.prototype.sites["youtube.com"].skip_link_extraction = function()
     return false;
 }
 
+LinternaMagica.prototype.sites["youtube.com"].skip_video_id_extraction = function()
+{
+    this.log("LinternaMagica.sites.skip_video_id_extraction:\n"+
+	     "Skipping video_id extraction in YouTube.",4);
+    return false;
+}
+
+
 // Extracts data for the flash object in youtube from a script
 LinternaMagica.prototype.sites["youtube.com"].extract_object_from_script =
 function()
@@ -343,8 +358,8 @@ function()
 
     this.extract_video_id_data = data;
 
-    var fmt = this.extract_youtube_fmt_parameter(data);
-    var maps = this.extract_youtube_fmt_url_map(data);
+    var fmt = this.extract_youtube_fmt_parameter();
+    var maps = this.extract_youtube_fmt_url_map();
 
     var hd_links = this.create_youtube_links(fmt, maps);
     var link = (hd_links && hd_links.length) ? hd_links : null;
@@ -372,6 +387,16 @@ function()
 	embed_id= embed_id[embed_id.length-1];
     }
 
+    var p = this.get_first_element_by_class("flash-player");
+    p = p ? p : this.get_first_element_by_class("html5-player");
+
+
+    if (!width || !height)
+    {
+	height = p ? p.clientHeight : null;
+	width  = p ? p.clientWidth : null;
+    }
+
     if (!width || !height)
     {
 	this.log("LinternaMagica.extract_object_from_script_youtube:\n"+
@@ -419,7 +444,7 @@ function()
 	linterna_magica_id =
 	    this.mark_flash_object("extracted-from-script");
 
-	object_data.parent = document.getElementById("watch-player");
+	object_data.parent =  p;
     }
 
     object_data.linterna_magica_id = linterna_magica_id;
@@ -455,7 +480,7 @@ function(object_data)
 	this.youtube_flash_upgrade_timeout = setInterval(
 	    function() {
 		self.detect_youtube_flash_upgrade.apply(self,[data]);
-	    }, 500);
+	    }, 2000);
     }
     
     return false;
@@ -464,6 +489,27 @@ function(object_data)
 LinternaMagica.prototype.sites["youtube.com"].css_fixes =
 function(object_data)
 {
+
+    // Sometimes when flash is installed the flash video object does
+    // not have (at all or the right one) linterna_magica_id. Usually
+    // the other objects if any are useless. This renders both LM and
+    // the flash interface. The code bellow tries to avoid it. Reason
+    // *unknown*.
+
+    this.log("LinternaMagica.youtube.css_fixes:\n "+
+	     "Harvesting (possible) lost flash video object with "+
+	     "linterna_magica_id "+ object_data.linterna_magica_id);
+
+    var movie_player = document.getElementById('movie_player');
+    if (movie_player) {
+	movie_player.linterna_magica_id = object_data.linterna_magica_id;
+    }
+
+    if (this.priority.self > this.priority.plugin)
+    {
+	this.hide_flash_video_object(object_data.linterna_magica_id);
+    }
+
     if (document.getElementById("playnav-playview"))
     {
 	// In channels/user pages in YouTube the web controlls are
@@ -590,6 +636,42 @@ function(object_data)
 	    // will not be displaced.
 	    html5_wrapper.style.setProperty('display', "none", "important");
 
+	    // If the HTML5 player has a fallback CSS class and LM can
+	    // access it it means there is no other option for
+	    // playback. We take over the player.
+	    //
+	    // See bug 37630
+	    // https://savannah.nongnu.org/bugs/index.php?37630
+	    if (this.priority.html5 > this.priority.self)
+ 	    {
+		// It should be available, because it is used above in
+		// this function
+		id = id ? id : object_data.linterna_magica_id;
+
+		var lm = document.getElementById("linterna-magica-"+id);
+
+		if (!lm)
+		{
+		    return null;
+		}
+
+		lm.style.removeProperty("display");
+	    
+		// It should be available, because it is used above in
+		// this functon.
+		toggle_plugin = toggle_plugin  ? toggle_plugin :
+		    document.getElementById("linterna-magica-"+
+					    "toggle-plugin-"+id);
+
+		toggle_plugin.parentNode.removeChild(toggle_plugin);
+
+		toggle_plugin_header =
+		    document.getElementById("linterna-magica-"+
+					    "toggle-plugin-header-"+id);
+		
+		toggle_plugin_header.parentNode.removeChild(toggle_plugin_header);
+	    }
+	    
 	    // Hide site controls and video container. Should not
 	    // overlap with Linterna Magica now. Fixes the overlapping
 	    // of HTML5 player and LM.
@@ -611,3 +693,41 @@ function(object_data)
 
     return false;
 }
+
+// Fixes the overlapping of the HD links list in YouTube's new design
+// See bug #37881
+// http://savannah.nongnu.org/bugs/?37881
+LinternaMagica.prototype.sites["youtube.com"].post_show_hd_links_list =
+function()
+{
+    var play_list= document.getElementById("watch7-playlist-tray-container");
+    if (play_list)
+    {
+	play_list.style.setProperty("display", "none", "important");
+    }
+
+    var side_bar= document.getElementById("watch7-sidebar");
+    if (side_bar)
+    {
+	side_bar.style.setProperty("display", "none", "important");
+    }
+}
+
+// Fixes the overlapping of the HD links list in YouTube's new design
+// See bug #37881
+// http://savannah.nongnu.org/bugs/?37881
+LinternaMagica.prototype.sites["youtube.com"].post_hide_hd_links_list =
+function()
+{
+    var play_list= document.getElementById("watch7-playlist-tray-container");
+    if (play_list)
+    {
+	play_list.style.removeProperty("display");
+    }
+
+    var side_bar= document.getElementById("watch7-sidebar");
+    if (side_bar)
+    {
+	side_bar.style.removeProperty("display");
+    }
+}
